#!/usr/bin/env python3
#
# Copyright (c) 2024-2025 ABXR Labs, Inc.
# Released under the MIT License. See LICENSE file for details.
#

"""
Shared constants for the ArborXR CLI
"""

# System files and directories to exclude from bundle processing
SYSTEM_FILES_TO_EXCLUDE = {'.DS_Store', 'Thumbs.db'}
SYSTEM_DIRS_TO_EXCLUDE = {'__MACOSX'}
