#
# Copyright (c) 2024-2025 ABXR Labs, Inc.
# Released under the MIT License. See LICENSE file for details.
#

from importlib.metadata import version as _get_version
version = _get_version("abxrcli")
